/* DESFuser.c
 * Copyright 2001 Matthew J. Holmes
 */
 
#include <PalmOS.h>
#include <SystemMgr.h>
#include <FeatureMgr.h>
#include <Encrypt.h>
#include <AFFuserGlue.h>

// Enumeration of launch codes
typedef enum
{
    InitDES = 32768, // launch codes
	Encrypt,
	Decrypt,
	GetMem = 0,      // internal flags
	SetMem
}
LaunchCodes;

typedef struct
{    
	//MemHandle DbName;
	UInt32 DbName;
    //MemHandle DESKey;
    UInt32 DESKey;	
	UInt16 Index;
	Int16 Data;		// internal field
}
DESParam;

static Err DoDES(UInt16 cmd, DESParam *Ptr);
static Err DoFeature(UInt16 cmd, DESParam *Ptr);
static Err DES_Encrypt(DmOpenRef dbRef, UInt32 RecSize, DESParam *Ptr);
static Err DES_Decrypt(DmOpenRef dbRef, UInt32 RecSize, DESParam *Ptr);

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    Err RC = 1;

    switch(cmd) {
    case InitDES:
        RC = DoFeature(cmd, (DESParam *)cmdPBP);
        break;    
    
    case Encrypt:
    case Decrypt:
        RC = DoDES(cmd, (DESParam *)cmdPBP);
    }

    return RC;
}

static Err DoDES(UInt16 cmd, DESParam *Ptr)
{
	Err Rc = 1;
	UInt32 value;
    LocalID hId;
    DmOpenRef dbRef;
    MemHandle hRec;
    MemHandle hDbName;

    FtrGet(sysFtrCreator, sysFtrNumEncryption, &value);
    if (!(value & sysFtrNumEncryptionMaskDES))
        return Rc;
        
    hDbName = AFStringToMemHandle(Ptr->DbName);
    hId = DmFindDatabase(0, MemHandleLock(hDbName));
    MemHandleUnlock(hDbName);
    if (!hId)
    	return Rc;
    	
    dbRef = DmOpenDatabase(0, hId, dmModeReadWrite | dmModeShowSecret);
    if (!dbRef)
    	return Rc;
    	
    hRec = DmQueryRecord(dbRef, Ptr->Index);
    if (hRec == NULL)
    	goto close_db;
    	
    value = MemHandleSize(hRec);
    if (value == 0)
    	goto close_db;
    	
    if (cmd == Encrypt)
   		Rc = DES_Encrypt(dbRef, value, Ptr);
    else
    	Rc = DES_Decrypt(dbRef, value, Ptr);
    	
close_db:
	DmCloseDatabase(dbRef);
	
    return Rc;
}

static Err DES_Encrypt(DmOpenRef dbRef, UInt32 RecSize, DESParam *Ptr)
{
	UInt32 i;
    MemPtr pRec;
    MemPtr pKey;
    MemHandle hRec;
    MemHandle hDESKey;
    UInt8 Delta;
        
    // A feature setting other than -1 indicates record already encrypted!
	DoFeature(GetMem, Ptr);
	if (Ptr->Data != -1) 
		return 1;
		
	Delta = 0;
	if (RecSize % 8)
	{
		Delta = 8 - (RecSize % 8);
		RecSize += Delta;
		DmResizeRecord(dbRef, Ptr->Index, RecSize);			
	}

    hDESKey = AFStringToMemHandle(Ptr->DESKey);
	pKey = MemHandleLock(hDESKey);
	hRec = DmGetRecord(dbRef, Ptr->Index);
	pRec = MemHandleLock(hRec);
	
	for (i = 0; i < RecSize; i += 8)
	{
		UInt8 buffer[8];
	    	
		MemMove(buffer, (unsigned char *)pRec + i, 8);
		EncDES(buffer, pKey, buffer, true);
		DmWrite(pRec, i, buffer, 8);
	}
	
	MemHandleUnlock(hRec);
	MemHandleUnlock(hDESKey);
	DmReleaseRecord(dbRef, Ptr->Index, true);
	
	Ptr->Data = Delta;		
	DoFeature(SetMem, Ptr);	

    return 0;
}


static Err DES_Decrypt(DmOpenRef dbRef, UInt32 RecSize, DESParam *Ptr)
{
    UInt32 i;
    MemPtr pRec;
    MemPtr pKey;
    MemHandle hRec;
    MemHandle hDESKey;
    Int8 Delta ;
	
    // A feature setting of -1 indicates record is not encrypted!
	DoFeature(GetMem, Ptr);
	if (Ptr->Data == -1) 
		return 1;
		
	Delta = Ptr->Data;
	
	hDESKey = AFStringToMemHandle(Ptr->DESKey);
	pKey = MemHandleLock(hDESKey);
	hRec = DmGetRecord(dbRef, Ptr->Index);
	pRec = MemHandleLock(hRec);
	
	for (i = 0; i < RecSize; i += 8)
	{
		UInt8 buffer[8];
	    UInt8 amount = 8;
	    	
		MemMove(buffer, (unsigned char *)pRec + i, 8);
		EncDES(buffer, pKey, buffer, false);
						
		if ((i + 8) > (RecSize - Delta))
		    amount -= Delta;
		DmWrite(pRec, i, buffer, amount);
	}
	
	MemHandleUnlock(hRec);
	DmReleaseRecord(dbRef, Ptr->Index, true);
	MemHandleUnlock(hDESKey);
	
	if (Delta)
		DmResizeRecord(dbRef, Ptr->Index, RecSize - Delta);
		
	Ptr->Data = -1;		
	DoFeature(SetMem, Ptr);
		
    return 0;
}


static Err DoFeature(UInt16 cmd, DESParam *Ptr)
{
	Int8 byte;
    Int8 *chunk;
    
	#define CREATOR 'DESL'
	#define FEATURE 1    

    switch(cmd) {
    case InitDES:
        byte = -1;
        if (FtrPtrNew(CREATOR, FEATURE, Ptr->Index, (void **)&chunk) == 0)
            DmSet(chunk, 0, Ptr->Index, byte);
        break;

    case SetMem:
        FtrGet(CREATOR, FEATURE, (UInt32 *)&chunk);
        byte = Ptr->Data;
        DmWrite(chunk, Ptr->Index, &byte, 1);
        break;

    case GetMem:
        FtrGet(CREATOR, FEATURE, (UInt32 *)&chunk);
        byte = *(chunk + Ptr->Index);
        Ptr->Data = byte;
        break;
    }

    return 0;
}