#ifndef __DBS_LIB_MEM_H__
#define __DBS_LIB_MEM_H__

// Palm OS common definitions
#include <PalmTypes.h>

// Library globals
typedef struct DBSLibGlobalMemType
{
	UInt16 refNum;		   // our library's reference number; OS assigned
	Int16  libRefCount;	// library reference count; number of times it
					         // has been opened by client applications
	Int16  clntRefCount; // number of client contexts we are servicing

    //Additional application specific globals
	
} DBSLibGlobalMemType;

typedef DBSLibGlobalMemType*		DBSLibGlobalMemPtr;


//Internal library states
#define dbsLibDBCreatePending       0x00
#define dbsLibQuerySchemaPending    0x02


// Client structure for storing client-specific data
//Field type definitions.  These map to corresponding VB data types
#define F_TYPE_BOOL   0  //Boolean
#define F_TYPE_BYTE   1  
#define F_TYPE_DATE   2
#define F_TYPE_FLOAT  3
#define F_TYPE_INT    4  //Int16
#define F_TYPE_LONG   5  //Int32
#define F_TYPE_STR    6  //Char

#define MAX_DATA_LEN 128
//Structure to maintain field definition information
typedef struct DBSLibFieldElemType
{
   Int32 nNum;                //Unique field number; for AppForge
   Char szName[DB_NAME_LEN];  //Field name
   Int16 nType;               //Field type; for AppForge
   UInt32 nDataLen;           //Size of field
   MemHandle hData;           //Field data
   Int32 nLength;             //Field length; for AppForge

} DBSLibFieldElemType;

typedef DBSLibFieldElemType* DBSLibFieldElemTypePtr;

//Client memory structure type
typedef struct DBSLibClientMemType
{
	Int16  nState;                  //Keep track of where we are
	Char   szName[DB_NAME_LEN];     //Name of the database we are creating
	Char   szSchema[DB_SCHEMA_LEN]; //Database schema, in "num type len ..." format
	Int16  nNumFields;              //Number of fields in each record
	Int16  nNumRecords;             //Number of records in the database
} DBSLibClientMemType;

typedef DBSLibClientMemType* DBSLibClientMemPtr;


/********************************************************************
 * Global memory management routines
 ********************************************************************/
extern DBSLibGlobalMemPtr CreateGlobalMem(UInt16 refNum);
extern void FreeGlobalMem(UInt16 refNum);
extern DBSLibGlobalMemPtr LockGlobalMem(UInt16 refNum);
extern Boolean IsDBSLibOpen(UInt16 refNum);
#define UnlockGlobalMem(gP) MemPtrUnlock(gP)

/********************************************************************
 * Client memory management routines
 ********************************************************************/
extern Err CreateClientMem(DBSLibGlobalMemPtr gP, UInt32 * clientContextP);
extern Err FreeClientMem(DBSLibGlobalMemPtr gP, UInt32 clientContext);
extern  DBSLibClientMemPtr LockClientMem(UInt32 context);
#define UnlockClientMem(contextP) MemPtrUnlock(contextP)


#endif //__DBS_LIB_MEM_H__